@@ -6,8 +6,10 @@
 
 import re
 
+from quodlibet import app
 from quodlibet.plugins.events import EventPlugin
 
+
 class RadioAdMute(EventPlugin):
     PLUGIN_ID = "radio_ad_mute"
     PLUGIN_NAME = _("Mute radio ads")
@@ -37,12 +39,11 @@ class RadioAdMute(EventPlugin):
         self.plugin_on_song_ended()
 
     def plugin_on_song_started(self, song):
-        from quodlibet.player import playlist as player
-
         # only check stream info songs
-        if player.song is player.info:
+        if not song or not song.streamsong:
             return
 
+        player = app.player
         data = song("~title~artist")
         for spam in self.SPAM:
             if spam.search(data):
@@ -56,8 +57,7 @@ class RadioAdMute(EventPlugin):
             return
         self.__muted = False
 
-        from quodlibet.player import playlist as player
-
+        player = app.player
         if player.volume != 0:  # volume changed, do nothing
             return
 
