@@ -12,16 +12,15 @@ import re
 import threading
 import time
 
-import gtk
-import gobject
-import pango
+from gi.repository import Gtk, GObject, Pango, GLib
 
 try:
     from musicbrainz2 import webservice as ws
     from musicbrainz2.utils import extractUuid
 except ImportError:
     from quodlibet import plugins
-    if not hasattr(plugins, "PluginImportException"): raise
+    if not hasattr(plugins, "PluginImportException"):
+        raise
     raise plugins.PluginImportException(
         "Couldn't find python-musicbrainz2.")
 
@@ -32,6 +31,7 @@ from quodlibet.qltk.views import HintedTreeView, MultiDragTreeView
 
 VARIOUS_ARTISTS_ARTISTID = '89ad4ac3-39f7-470e-963a-56509c546377'
 
+
 def get_artist(album):
     """Returns a single artist likely to be the MB AlbumArtist, or None."""
     for tag in ["albumartist", "artist", "performer"]:
@@ -44,48 +44,49 @@ def get_artist(album):
             return None
     return None
 
+
 def get_trackcount(album):
     """Returns the track count, hammered into submission."""
     return max(max(map(lambda t: max(map(int,
         t.get('tracknumber', '0').split('/'))), album)), len(album)) # (;))
 
+
 def config_get(key, default=''):
-    try:
-        return config.getboolean('plugins', 'brainz_' + key)
-    except config.error:
-        return default
+    return config.getboolean('plugins', 'brainz_' + key, default)
+
 
 def dialog_get_widget_for_stockid(dialog, stockid):
     for child in dialog.get_action_area().get_children():
         if child.get_label() == stockid:
             return child
 
+
 class ResultTreeView(HintedTreeView, MultiDragTreeView):
     """The result treeview. The model only stores local tracks; info about
     remote results is pulled from self.remote_album."""
 
-    def __name_datafunc(self, col, cell, model, itr):
+    def __name_datafunc(self, col, cell, model, itr, data):
         song = model[itr][0]
         if song:
             cell.set_property('text', os.path.basename(song.get("~filename")))
         else:
             cell.set_property('text', '')
 
-    def __track_datafunc(self, col, cell, model, itr):
+    def __track_datafunc(self, col, cell, model, itr, data):
         idx = model.get_path(itr)[0]
         if idx >= len(self.remote_album):
             cell.set_property('text', '')
         else:
-            cell.set_property('text', idx + 1)
+            cell.set_property('text', str(idx + 1))
 
-    def __title_datafunc(self, col, cell, model, itr):
+    def __title_datafunc(self, col, cell, model, itr, data):
         idx = model.get_path(itr)[0]
         if idx >= len(self.remote_album):
             cell.set_property('text', '')
         else:
             cell.set_property('text', self.remote_album[idx].title)
 
-    def __artist_datafunc(self, col, cell, model, itr):
+    def __artist_datafunc(self, col, cell, model, itr, data):
         idx = model.get_path(itr)[0]
         if idx >= len(self.remote_album) or not self.remote_album[idx].artist:
             cell.set_property('text', '')
@@ -95,14 +96,14 @@ class ResultTreeView(HintedTreeView, MultiDragTreeView):
     def __init__(self, album):
         self.album = album
         self.remote_album = []
-        self.model = gtk.ListStore(object)
+        self.model = Gtk.ListStore(object)
         map(self.model.append, zip(album))
 
         super(ResultTreeView, self).__init__(self.model)
         self.set_headers_clickable(True)
         self.set_rules_hint(True)
         self.set_reorderable(True)
-        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
 
         cols = [
                 ('Filename', self.__name_datafunc, True),
@@ -112,9 +113,9 @@ class ResultTreeView(HintedTreeView, MultiDragTreeView):
             ]
 
         for title, func, resize in cols:
-            render = gtk.CellRendererText()
-            render.set_property('ellipsize', pango.ELLIPSIZE_END)
-            col = gtk.TreeViewColumn(title, render)
+            render = Gtk.CellRendererText()
+            render.set_property('ellipsize', Pango.EllipsizeMode.END)
+            col = Gtk.TreeViewColumn(title, render)
             col.set_cell_data_func(render, func)
             col.set_resizable(resize)
             col.set_expand(resize)
@@ -134,19 +135,22 @@ class ResultTreeView(HintedTreeView, MultiDragTreeView):
         has_artists = bool(filter(lambda t: t.artist, remote_album))
         col = self.get_column(3)
         # sometimes gets called after the treeview is already gone
-        if not col: return
+        if not col:
+            return
         col.set_visible(has_artists)
         self.columns_autosize()
         self.queue_draw()
 
-class ResultComboBox(gtk.ComboBox):
+
+class ResultComboBox(Gtk.ComboBox):
     """Formatted picker for different Result entries."""
 
     def __init__(self, model):
-        super(ResultComboBox, self).__init__(model)
-        render = gtk.CellRendererText()
+        super(ResultComboBox, self).__init__(model=model)
+        render = Gtk.CellRendererText()
         render.set_fixed_height_from_font(2)
-        def celldata(layout, cell, model, iter):
+
+        def celldata(layout, cell, model, iter, data):
             release = model[iter][0]
             if not release:
                 return
@@ -160,25 +164,26 @@ class ResultComboBox(gtk.ComboBox):
                     util.escape(release.artist.name),
                     date, release.tracksCount)
             cell.set_property('markup', markup)
-        self.pack_start(render)
-        self.set_cell_data_func(render, celldata)
+        self.pack_start(render, True)
+        self.set_cell_data_func(render, celldata, None)
 
-class ReleaseEventComboBox(gtk.HBox):
+
+class ReleaseEventComboBox(Gtk.HBox):
     """A ComboBox for picking a release event."""
 
     def __init__(self):
         super(ReleaseEventComboBox, self).__init__()
-        self.model = gtk.ListStore(object, str)
-        self.combo = gtk.ComboBox(self.model)
-        render = gtk.CellRendererText()
-        self.combo.pack_start(render)
-        self.combo.set_attributes(render, markup=1)
+        self.model = Gtk.ListStore(object, str)
+        self.combo = Gtk.ComboBox(model=self.model)
+        render = Gtk.CellRendererText()
+        self.combo.pack_start(render, True)
+        self.combo.add_attribute(render, "markup", 1)
         self.combo.set_sensitive(False)
-        self.label = gtk.Label("_Release:")
+        self.label = Gtk.Label(label="_Release:", use_underline=True)
         self.label.set_use_underline(True)
         self.label.set_mnemonic_widget(self.combo)
-        self.pack_start(self.label, expand=False)
-        self.pack_start(self.combo)
+        self.pack_start(self.label, False, True, 0)
+        self.pack_start(self.combo, True, True, 0)
 
     def update(self, release):
         self.model.clear()
@@ -192,8 +197,8 @@ class ReleaseEventComboBox(gtk.HBox):
         for rel_event in events:
             text = '%s %s: <b>%s</b> <i>(%s)</i>' % (
                     rel_event.getDate() or '', rel_event.getLabel() or '',
-                    rel_event.getCatalogNumber(),rel_event.getCountry())
-            self.model.append( (rel_event, text) )
+                    rel_event.getCatalogNumber(), rel_event.getCountry())
+            self.model.append((rel_event, text))
         if len(events) > 0:
             self.combo.set_active(0)
         self.combo.set_sensitive((len(events) > 0))
@@ -208,6 +213,7 @@ class ReleaseEventComboBox(gtk.HBox):
         else:
             return None
 
+
 class QueryThread:
     """Daemon thread which does HTTP retries and avoids flooding."""
     def __init__(self):
@@ -238,15 +244,15 @@ class QueryThread:
                         res = func(*args, **kwargs)
                     except:
                         res = None
-                gobject.idle_add(callback, res)
+                GLib.idle_add(callback, res)
             time.sleep(1)
 
 
-class SearchWindow(gtk.Dialog):
+class SearchWindow(Gtk.Dialog):
     def __save(self, widget=None, response=None):
         """Writes values to Song objects."""
         self._qthread.stop()
-        if response != gtk.RESPONSE_ACCEPT:
+        if response != Gtk.ResponseType.ACCEPT:
             self.destroy()
             return
 
@@ -285,12 +291,14 @@ class SearchWindow(gtk.Dialog):
             shared['musicbrainz_albumid'] = extractUuid(album.id)
 
         for idx, (song, ) in enumerate(self.result_treeview.model):
-            if song is None: continue
+            if song is None:
+                continue
             song.update(shared)
-            if idx >= len(album.tracks): continue
+            if idx >= len(album.tracks):
+                continue
             track = album.tracks[idx]
             song['title'] = track.title
-            song['tracknumber'] = '%d/%d' % (idx+1,
+            song['tracknumber'] = '%d/%d' % (idx + 1,
                     max(len(album.tracks), len(self.result_treeview.model)))
             if config_get('standard', True):
                 song['musicbrainz_trackid'] = extractUuid(track.id)
@@ -351,7 +359,8 @@ class SearchWindow(gtk.Dialog):
     def __result_changed(self, combo):
         """Called when a release is chosen from the result combo."""
         idx = combo.get_active()
-        if idx == -1: return
+        if idx == -1:
+            return
         rel_id = self._resultlist[idx][0].id
         if rel_id in self._releasecache:
             self.__update_results(self._releasecache[rel_id])
@@ -367,37 +376,40 @@ class SearchWindow(gtk.Dialog):
         num_results = len(self._resultlist)
         text = ngettext("Found %d result.", "Found %d results.", num_results)
         self.result_label.set_text(text % num_results)
+        # issue 973: search can return invalid (or removed) ReleaseIDs
+        if release is None:
+            return
         self._releasecache.setdefault(extractUuid(release.id), release)
         self.result_treeview.update_remote_album(release.tracks)
         self.current_release = release
         self.release_combo.update(release)
-        save_button = dialog_get_widget_for_stockid(self, gtk.STOCK_SAVE)
+        save_button = dialog_get_widget_for_stockid(self, Gtk.STOCK_SAVE)
         save_button.set_sensitive(True)
 
     def __init__(self, album, cache):
         self.album = album
 
         self._query = ws.Query()
-        self._resultlist = gtk.ListStore(gobject.TYPE_PYOBJECT)
+        self._resultlist = Gtk.ListStore(GObject.TYPE_PYOBJECT)
         self._releasecache = cache
         self._qthread = QueryThread()
         self.current_release = None
 
         super(SearchWindow, self).__init__("MusicBrainz lookup", buttons=(
-                    gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
-                    gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))
+                    Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
+                    Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
         self.set_default_size(650, 500)
         self.set_border_width(5)
 
-        save_button = dialog_get_widget_for_stockid(self, gtk.STOCK_SAVE)
+        save_button = dialog_get_widget_for_stockid(self, Gtk.STOCK_SAVE)
         save_button.set_sensitive(False)
 
-        vb = gtk.VBox()
+        vb = Gtk.VBox()
         vb.set_spacing(8)
 
-        hb = gtk.HBox()
+        hb = Gtk.HBox()
         hb.set_spacing(8)
-        sq = self.search_query = gtk.Entry()
+        sq = self.search_query = Gtk.Entry()
         sq.connect('activate', self.__do_query)
 
         alb = '"%s"' % album[0].comma("album").replace('"', '')
@@ -405,44 +417,44 @@ class SearchWindow(gtk.Dialog):
         if art:
             alb = '%s AND artist:"%s"' % (alb, art.replace('"', ''))
         sq.set_text('%s AND tracks:%d' %
-                (alb, get_trackcount(album)) )
+                (alb, get_trackcount(album)))
 
-        lbl = gtk.Label("_Query:")
+        lbl = Gtk.Label(label="_Query:")
         lbl.set_use_underline(True)
         lbl.set_mnemonic_widget(sq)
-        stb = self.search_button = gtk.Button('S_earch')
+        stb = self.search_button = Gtk.Button('S_earch', use_underline=True)
         stb.connect('clicked', self.__do_query)
-        hb.pack_start(lbl, expand=False)
-        hb.pack_start(sq)
-        hb.pack_start(stb, expand=False)
-        vb.pack_start(hb, expand=False)
+        hb.pack_start(lbl, False, True, 0)
+        hb.pack_start(sq, True, True, 0)
+        hb.pack_start(stb, False, True, 0)
+        vb.pack_start(hb, False, True, 0)
 
         self.result_combo = ResultComboBox(self._resultlist)
         self.result_combo.connect('changed', self.__result_changed)
-        vb.pack_start(self.result_combo, expand=False)
+        vb.pack_start(self.result_combo, False, True, 0)
 
-        rhb = gtk.HBox()
-        rl = gtk.Label()
+        rhb = Gtk.HBox()
+        rl = Gtk.Label()
         rl.set_markup("Results <i>(drag to reorder)</i>")
         rl.set_alignment(0, 0.5)
-        rhb.pack_start(rl, expand=False)
-        rl = self.result_label = gtk.Label("")
-        rhb.pack_end(rl, expand=False)
-        vb.pack_start(rhb, expand=False)
-        sw = gtk.ScrolledWindow()
-        sw.set_shadow_type(gtk.SHADOW_IN)
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+        rhb.pack_start(rl, False, True, 0)
+        rl = self.result_label = Gtk.Label(label="")
+        rhb.pack_end(rl, False, True, 0)
+        vb.pack_start(rhb, False, True, 0)
+        sw = Gtk.ScrolledWindow()
+        sw.set_shadow_type(Gtk.ShadowType.IN)
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.ALWAYS)
         rtv = self.result_treeview = ResultTreeView(self.album)
         rtv.set_border_width(8)
         sw.add(rtv)
-        vb.pack_start(sw)
+        vb.pack_start(sw, True, True, 0)
 
-        hb = gtk.HBox()
+        hb = Gtk.HBox()
         hb.set_spacing(8)
         self.release_combo = ReleaseEventComboBox()
-        vb.pack_start(self.release_combo, expand=False)
+        vb.pack_start(self.release_combo, False, True, 0)
 
-        self.get_content_area().pack_start(vb)
+        self.get_content_area().pack_start(vb, True, True, 0)
         self.connect('response', self.__save)
 
         stb.emit('clicked')
@@ -452,7 +464,7 @@ class SearchWindow(gtk.Dialog):
 class MyBrainz(SongsMenuPlugin):
     PLUGIN_ID = "MusicBrainz lookup"
     PLUGIN_NAME = "MusicBrainz Lookup"
-    PLUGIN_ICON = gtk.STOCK_CDROM
+    PLUGIN_ICON = Gtk.STOCK_CDROM
     PLUGIN_DESC = 'Retag an album based on a MusicBrainz search.'
     PLUGIN_VERSION = '0.5'
 
@@ -465,7 +477,7 @@ class MyBrainz(SongsMenuPlugin):
                 discnum = int(song.get('discnumber', '1').split('/')[0])
                 discs.setdefault(discnum, []).append(song)
             for disc in discs.values():
-                s = SearchWindow(disc, self.cache).run()
+                SearchWindow(disc, self.cache).run()
 
     @classmethod
     def PluginPreferences(self, win):
@@ -479,14 +491,12 @@ class MyBrainz(SongsMenuPlugin):
             ('labelid', 'Write _labelid tag (fixes multi-disc albums)', True),
         ]
 
-        vb = gtk.VBox()
+        vb = Gtk.VBox()
         vb.set_spacing(8)
 
         for key, label, default in items:
             ccb = ConfigCheckButton(label, 'plugins', 'brainz_' + key)
             ccb.set_active(config_get(key, default))
-            vb.pack_start(ccb)
+            vb.pack_start(ccb, True, True, 0)
 
         return vb
-
-
