@@ -1,27 +1,42 @@
 # -*- coding: utf-8 -*-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# Copyright (C) 2006 - Steve Frécinaux
 #
-# Slightly modified version from totem (console.py)
-#     Copyright (C) 2006 - Steve Frécinaux
-# Parts from "Interactive Python-GTK Console" (from epiphany's console.py)
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+# Parts from "Interactive Python-GTK Console"
+# (stolen from epiphany's console.py)
 #     Copyright (C), 1998 James Henstridge <james@daa.com.au>
 #     Copyright (C), 2005 Adam Hooper <adamh@densi.com>
 # Bits from gedit Python Console Plugin
 #     Copyrignt (C), 2005 Raphaël Slinckx
 
+# PythonConsole taken from totem
+# Plugin parts:
+# Copyright 2009,2010,2013 Christoph Reiter
+
+
 import sys
 import re
 import traceback
 
-import gobject
-import pango
-import gtk
+from gi.repository import Gtk, Pango, Gdk, GLib
 
 from quodlibet import qltk, const
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 
+
 class PyConsole(SongsMenuPlugin):
     PLUGIN_ID = 'Python Console'
     PLUGIN_NAME = _('Python Console')
@@ -30,51 +45,66 @@ class PyConsole(SongsMenuPlugin):
     PLUGIN_VERSION = '0.2'
 
     def plugin_songs(self, songs):
+        win = ConsoleWindow(songs)
+        win.set_icon_name(self.PLUGIN_ICON)
+        win.set_title(self.PLUGIN_DESC + " (Quod Libet)")
+        win.show_all()
+
+
+class ConsoleWindow(Gtk.Window):
+    def __init__(self, songs):
+        Gtk.Window.__init__(self)
+
         files = [song('~filename') for song in songs]
         song_dicts = [song._song for song in songs]
 
-        self.win = win = qltk.Window()
-        win.set_size_request(600, 400)
-        win.set_icon_name(self.PLUGIN_ICON)
-        win.set_title(self.PLUGIN_DESC + " (Quod Libet)")
+        self.set_size_request(600, 400)
 
+        from quodlibet import app
         console = PythonConsole(
-            namespace = {
-                'songs' : songs,
+            namespace={
+                'songs': songs,
                 'files': files,
-                'sdict': song_dicts})
-        win.add(console)
-        win.show_all()
+                'sdict': song_dicts,
+                'app': app})
+        self.add(console)
 
         acces_string = _("You can access the following objects by default:\\n"
             "  '%s' (SongWrapper objects)\\n"
             "  '%s' (Song dictionaries)\\n"
-            "  '%s' (Filename list)") % ("songs", "sdict", "files")
+            "  '%s' (Filename list)\\n"
+            "  '%s' (Application instance)") % (
+            "songs", "sdict", "files", "app")
 
         dir_string = _("Your current working directory is:")
 
+        console.eval("import mutagen", False)
         console.eval("import os", False)
         console.eval("print \"Python: %s / Quod Libet: %s\"" %
             (sys.version.split()[0], const.VERSION), False)
         console.eval("print \"%s\"" % acces_string, False)
         console.eval("print \"%s \"+ os.getcwd()" % dir_string, False)
 
-class PythonConsole(gtk.ScrolledWindow):
-    def __init__(self, namespace = {}):
-        gtk.ScrolledWindow.__init__(self)
+        console.connect("destroy", lambda *x: self.destroy())
 
-        self.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        self.set_shadow_type(gtk.SHADOW_IN)
-        self.view = gtk.TextView()
-        self.view.modify_font(pango.FontDescription('Monospace'))
+
+class PythonConsole(Gtk.ScrolledWindow):
+    def __init__(self, namespace={}, destroy_cb=None):
+        Gtk.ScrolledWindow.__init__(self)
+
+        self.destroy_cb = destroy_cb
+        self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+        self.set_shadow_type(Gtk.ShadowType.IN)
+        self.view = Gtk.TextView()
+        self.view.modify_font(Pango.font_description_from_string('Monospace'))
         self.view.set_editable(True)
-        self.view.set_wrap_mode(gtk.WRAP_WORD_CHAR)
+        self.view.set_wrap_mode(Gtk.WrapMode.CHAR)
         self.add(self.view)
         self.view.show()
 
         buffer = self.view.get_buffer()
         self.normal = buffer.create_tag("normal")
-        self.error  = buffer.create_tag("error")
+        self.error = buffer.create_tag("error")
         self.error.set_property("foreground", "red")
         self.command = buffer.create_tag("command")
         self.command.set_property("foreground", "blue")
@@ -103,21 +133,22 @@ class PythonConsole(gtk.ScrolledWindow):
         self.view.connect("key-press-event", self.__key_press_event_cb)
         buffer.connect("mark-set", self.__mark_set_cb)
 
-
     def __key_press_event_cb(self, view, event):
-        if event.keyval == gtk.keysyms.c and \
-            event.state & gtk.gdk.CONTROL_MASK:
-            self.set_command_line("")
+        modifier_mask = Gtk.accelerator_get_default_mod_mask()
+        event_state = event.state & modifier_mask
 
-        elif (event.keyval == gtk.keysyms.Return or \
-            event.keyval == gtk.keysyms.KP_Enter) and \
-            event.state & gtk.gdk.CONTROL_MASK:
+        if event.keyval == Gdk.KEY_d and \
+                event_state == Gdk.ModifierType.CONTROL_MASK:
+            self.destroy()
+
+        elif event.keyval == Gdk.KEY_Return and \
+                event_state == Gdk.ModifierType.CONTROL_MASK:
             # Get the command
             buffer = view.get_buffer()
             inp_mark = buffer.get_mark("input")
             inp = buffer.get_iter_at_mark(inp_mark)
             cur = buffer.get_end_iter()
-            line = buffer.get_text(inp, cur)
+            line = buffer.get_text(inp, cur, True)
             self.current_command = self.current_command + line + "\n"
             self.history_add(line)
 
@@ -130,15 +161,14 @@ class PythonConsole(gtk.ScrolledWindow):
             # Keep indentation of precendent line
             spaces = re.match(self.__spaces_pattern, line)
             if spaces is not None:
-                buffer.insert(cur, line[spaces.start() : spaces.end()])
+                buffer.insert(cur, line[spaces.start():spaces.end()])
                 cur = buffer.get_end_iter()
 
             buffer.place_cursor(cur)
-            gobject.idle_add(self.scroll_to_end)
+            GLib.idle_add(self.scroll_to_end)
             return True
 
-        elif event.keyval == gtk.keysyms.Return or \
-            event.keyval == gtk.keysyms.KP_Enter:
+        elif event.keyval == Gdk.KEY_Return:
             # Get the marks
             buffer = view.get_buffer()
             lin_mark = buffer.get_mark("input-line")
@@ -147,7 +177,7 @@ class PythonConsole(gtk.ScrolledWindow):
             # Get the command line
             inp = buffer.get_iter_at_mark(inp_mark)
             cur = buffer.get_end_iter()
-            line = buffer.get_text(inp, cur)
+            line = buffer.get_text(inp, cur, True)
             self.current_command = self.current_command + line + "\n"
             self.history_add(line)
 
@@ -179,38 +209,36 @@ class PythonConsole(gtk.ScrolledWindow):
             cur = buffer.get_end_iter()
             buffer.move_mark(inp_mark, cur)
             buffer.place_cursor(cur)
-            gobject.idle_add(self.scroll_to_end)
+            GLib.idle_add(self.scroll_to_end)
             return True
 
-        elif event.keyval == gtk.keysyms.KP_Down or \
-            event.keyval == gtk.keysyms.Down:
+        elif event.keyval == Gdk.KEY_KP_Down or event.keyval == Gdk.KEY_Down:
             # Next entry from history
             view.emit_stop_by_name("key_press_event")
             self.history_down()
-            gobject.idle_add(self.scroll_to_end)
+            GLib.idle_add(self.scroll_to_end)
             return True
 
-        elif event.keyval == gtk.keysyms.KP_Up or \
-            event.keyval == gtk.keysyms.Up:
+        elif event.keyval == Gdk.KEY_KP_Up or event.keyval == Gdk.KEY_Up:
             # Previous entry from history
             view.emit_stop_by_name("key_press_event")
             self.history_up()
-            gobject.idle_add(self.scroll_to_end)
+            GLib.idle_add(self.scroll_to_end)
             return True
 
-        elif event.keyval == gtk.keysyms.KP_Left or \
-            event.keyval == gtk.keysyms.Left or \
-            event.keyval == gtk.keysyms.BackSpace:
+        elif event.keyval == Gdk.KEY_KP_Left or \
+                event.keyval == Gdk.KEY_Left or \
+                event.keyval == Gdk.KEY_BackSpace:
             buffer = view.get_buffer()
             inp = buffer.get_iter_at_mark(buffer.get_mark("input"))
             cur = buffer.get_iter_at_mark(buffer.get_insert())
             return inp.compare(cur) == 0
 
-        elif event.keyval == gtk.keysyms.Home:
+        elif event.keyval == Gdk.KEY_Home:
             # Go to the begin of the command instead of the begin of the line
             buffer = view.get_buffer()
             inp = buffer.get_iter_at_mark(buffer.get_mark("input"))
-            if event.state == gtk.gdk.SHIFT_MASK:
+            if event_state == Gdk.ModifierType.SHIFT_MASK:
                 buffer.move_mark_by_name("insert", inp)
             else:
                 buffer.place_cursor(inp)
@@ -218,14 +246,14 @@ class PythonConsole(gtk.ScrolledWindow):
 
     def __mark_set_cb(self, buffer, iter, name):
         input = buffer.get_iter_at_mark(buffer.get_mark("input"))
-        pos   = buffer.get_iter_at_mark(buffer.get_insert())
+        pos = buffer.get_iter_at_mark(buffer.get_insert())
         self.view.set_editable(pos.compare(input) != -1)
 
     def get_command_line(self):
         buffer = self.view.get_buffer()
         inp = buffer.get_iter_at_mark(buffer.get_mark("input"))
         cur = buffer.get_end_iter()
-        return buffer.get_text(inp, cur)
+        return buffer.get_text(inp, cur, True)
 
     def set_command_line(self, command):
         buffer = self.view.get_buffer()
@@ -234,8 +262,8 @@ class PythonConsole(gtk.ScrolledWindow):
         cur = buffer.get_end_iter()
         buffer.delete(inp, cur)
         buffer.insert(inp, command)
-        buffer.select_range(
-            buffer.get_iter_at_mark(mark), buffer.get_end_iter())
+        buffer.select_range(buffer.get_iter_at_mark(mark),
+                            buffer.get_end_iter())
         self.view.grab_focus()
 
     def history_add(self, line):
@@ -258,18 +286,19 @@ class PythonConsole(gtk.ScrolledWindow):
 
     def scroll_to_end(self):
         iter = self.view.get_buffer().get_end_iter()
-        self.view.scroll_to_iter(iter, 0.0)
-        self.view.emit('move-cursor', gtk.MOVEMENT_BUFFER_ENDS, 1, False)
+        self.view.scroll_to_iter(iter, 0.0, False, 0.5, 0.5)
+        return False
 
-    def write(self, text, tag = None):
-        buffer = self.view.get_buffer()
+    def write(self, text, tag=None):
+        buf = self.view.get_buffer()
         if tag is None:
-            buffer.insert(buffer.get_end_iter(), text)
+            buf.insert(buf.get_end_iter(), text)
         else:
-            buffer.insert_with_tags(buffer.get_end_iter(), text, tag)
-        gobject.idle_add(self.scroll_to_end)
+            buf.insert_with_tags(buf.get_end_iter(), text, tag)
 
-    def eval(self, command, display_command = False):
+        GLib.idle_add(self.scroll_to_end)
+
+    def eval(self, command, display_command=False):
         buffer = self.view.get_buffer()
         lin = buffer.get_mark("input-line")
         buffer.delete(buffer.get_iter_at_mark(lin),
@@ -290,7 +319,7 @@ class PythonConsole(gtk.ScrolledWindow):
         buffer.insert(cur, ">>> ")
         cur = buffer.get_end_iter()
         buffer.move_mark_by_name("input", cur)
-        self.view.scroll_to_iter(buffer.get_end_iter(), 0.0)
+        self.view.scroll_to_iter(buffer.get_end_iter(), 0.0, False, 0.5, 0.5)
 
     def __run(self, command):
         sys.stdout, self.stdout = self.stdout, sys.stdout
@@ -300,31 +329,58 @@ class PythonConsole(gtk.ScrolledWindow):
             try:
                 r = eval(command, self.namespace, self.namespace)
                 if r is not None:
-                    print `r`
+                    print repr(r)
             except SyntaxError:
                 exec command in self.namespace
         except:
-            traceback.print_exc()
+            if hasattr(sys, 'last_type') and sys.last_type == SystemExit:
+                self.destroy()
+            else:
+                traceback.print_exc()
 
         sys.stdout, self.stdout = self.stdout, sys.stdout
         sys.stderr, self.stderr = self.stderr, sys.stderr
 
-class OutFile:
+
+class OutFile(object):
     """A fake output file object. It sends output to a TK test widget,
     and if asked for a file number, returns one set on instance creation"""
     def __init__(self, console, fn, tag):
         self.fn = fn
         self.console = console
         self.tag = tag
-    def close(self):         pass
-    def flush(self):         pass
-    def fileno(self):        return self.fn
-    def isatty(self):        return 0
-    def read(self, a):       return ''
-    def readline(self):      return ''
-    def readlines(self):     return []
-    def write(self, s):      self.console.write(s, self.tag)
-    def writelines(self, l): self.console.write(l, self.tag)
-    def seek(self, a):       raise IOError, (29, 'Illegal seek')
-    def tell(self):          raise IOError, (29, 'Illegal seek')
+
+    def close(self):
+        pass
+
+    def flush(self):
+        pass
+
+    def fileno(self):
+        return self.fn
+
+    def isatty(self):
+        return 0
+
+    def read(self, a):
+        return ''
+
+    def readline(self):
+        return ''
+
+    def readlines(self):
+        return []
+
+    def write(self, s):
+        self.console.write(s, self.tag)
+
+    def writelines(self, l):
+        self.console.write(l, self.tag)
+
+    def seek(self, a):
+        raise IOError(29, 'Illegal seek')
+
+    def tell(self):
+        raise IOError(29, 'Illegal seek')
+
     truncate = tell
