@@ -11,21 +11,21 @@
 #    it under the terms of version 2 of the GNU General Public License as
 #    published by the Free Software Foundation.
 #
-from gettext import ngettext
-from quodlibet import config, player, print_d, print_w, util, qltk
-from quodlibet.library import library
+
+import string
+import unicodedata
+
+from gi.repository import Gtk, Pango
+
+from quodlibet import app
+from quodlibet import print_d, print_w, util, qltk
+from quodlibet.plugins import PluginConfigMixin
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
+from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.edittags import AudioFileGroup
 from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.views import RCMHintedTreeView
-import ConfigParser
-import gobject
-import gtk
-import pango
-import string
-import unicodedata
-from quodlibet.qltk.ccb import ConfigCheckButton
 
 
 class DuplicateSongsView(RCMHintedTreeView):
@@ -33,21 +33,25 @@ class DuplicateSongsView(RCMHintedTreeView):
 
     def get_selected_songs(self):
         selection = self.get_selection()
-        if selection is None: return []
+        if selection is None:
+            return []
         model, rows = selection.get_selected_rows()
-        if not rows: return []
+        if not rows:
+            return []
         selected = []
         for row in rows:
-            if model[row][0]: selected.append(model[row][0])
-            else:
-                # TODO: reflect child-selections in GUI
-                for child in model[row].iterchildren():
+            row = model[row]
+            if row.parent is None:
+                for child in row.iterchildren():
                     selected.append(child[0])
+            else:
+                selected.append(row[0])
         return selected
 
     def Menu(self, library):
         songs = self.get_selected_songs()
-        if not songs: return
+        if not songs:
+            return
 
         menu = SongsMenu(
             library, songs, delete=True, parent=self, plugins=False,
@@ -55,21 +59,23 @@ class DuplicateSongsView(RCMHintedTreeView):
         menu.show_all()
         return menu
 
-    def __select_song(self, player, indices, col):
-        songs = self.get_selected_songs()
-        #print_d("Trying to play %s" % songs[0]("~artist~title~version"), self)
-        if player.go_to(songs[0], True):
-            player.paused = False
+    def __select_song(self, player, path, col):
+        if len(path) == 1:
+            if self.row_expanded(path):
+                self.collapse_row(path)
+            else:
+                self.expand_row(path, False)
         else:
-            print_w("Sorry, can't play song outside current list.")
-            # This is evil. EVIL. But it half-works.
-#            player.setup(self.get_model(), songs[0], 0)
-#            if player.go_to(songs[0], True):
-#                player.paused = False
+            songs = self.get_selected_songs()
+            if songs and player.go_to(songs[0], True):
+                player.paused = False
+            else:
+                print_w("Sorry, can't play song outside current list.")
 
     def _removed(self, library, songs):
         model = self.get_model()
-        if not model: return
+        if not model:
+            return
         for song in songs:
             row = model.find_row(song)
             if row:
@@ -86,7 +92,8 @@ class DuplicateSongsView(RCMHintedTreeView):
 
     def _added(self, library, songs):
         model = self.get_model()
-        if not model: return
+        if not model:
+            return
         for song in songs:
             key = Duplicates.get_key(song)
             model.add_to_existing_group(key, song)
@@ -121,9 +128,9 @@ class DuplicateSongsView(RCMHintedTreeView):
     def __init__(self, model):
         super(DuplicateSongsView, self).__init__(model)
         self.connect_object('row-activated',
-                            self.__select_song, player.playlist)
+                            self.__select_song, app.player)
         # Selecting multiple is a nice feature it turns out.
-        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
 
         # Handle signals propagated from the underlying library
         self.connected_library_sigs = []
@@ -134,23 +141,24 @@ class DuplicateSongsView(RCMHintedTreeView):
         }
         for (sig, callback) in SIGNAL_MAP.items():
             print_d("Listening to library.%s signals" % sig)
-            self.connected_library_sigs.append(library.connect(sig, callback))
+            self.connected_library_sigs.append(
+                app.library.connect(sig, callback))
 
         # And disconnect, or Bad Stuff happens.
         self.connect('destroy', self.on_destroy)
 
-
     def on_destroy(self, view):
         print_d("Disconnecting from library signals...")
         for sig in self.connected_library_sigs:
-            library.disconnect(sig)
+            app.library.disconnect(sig)
 
 
-class DuplicatesTreeModel(gtk.TreeStore):
+class DuplicatesTreeModel(Gtk.TreeStore):
     """A tree store to model duplicated song information"""
 
     # Define columns to display (and how, in lieu of using qltk.browsers)
-    def i(x): return x
+    def i(x):
+        return x
     TAG_MAP = [
         ("artist", i), ("title", i), ("album", i),
         ("~#length", lambda s: util.format_time(int(s))),
@@ -158,7 +166,8 @@ class DuplicatesTreeModel(gtk.TreeStore):
         ("~filename", i)]
     # Now make a dict. This seems clunky.
     tag_functions = {}
-    for t, f in TAG_MAP: tag_functions[t] = f
+    for t, f in TAG_MAP:
+        tag_functions[t] = f
 
     @classmethod
     def group_value(cls, group, tag):
@@ -170,7 +179,7 @@ class DuplicatesTreeModel(gtk.TreeStore):
         else:
             try:
                 group_val = cls.tag_functions[tag](group_val)
-            except (ValueError, TypeError), e:
+            except (ValueError, TypeError):
                 pass
         return group_val.replace("\n", ", ")
 
@@ -185,13 +194,14 @@ class DuplicatesTreeModel(gtk.TreeStore):
         return None
 
     def add_to_existing_group(self, key, song):
-        """Tries to add a song to an existing group. Returns None if not able"""
+        """Tries to add a song to an existing group. Returns None if not able
+        """
         #print_d("Trying to add %s to group \"%s\"" % (song("~filename"), key))
         for parent in self:
             if key == parent[0]:
                 print_d("Found group", self)
                 return self.append(parent.iter, self.__make_row(song))
-            # TODO: update group 
+            # TODO: update group
         return None
 
     @classmethod
@@ -211,11 +221,10 @@ class DuplicatesTreeModel(gtk.TreeStore):
         for s in songs:
             self.append(parent, self.__make_row(s))
 
-
     def go_to(self, song, explicit=False):
         #print_d("Duplicates: told to go to %r" % song, context=self)
         self.__iter = None
-        if isinstance(song, gtk.TreeIter):
+        if isinstance(song, Gtk.TreeIter):
             self.__iter = song
             self.sourced = True
         elif not self.find_row(song):
@@ -232,21 +241,30 @@ class DuplicatesTreeModel(gtk.TreeStore):
 
     @property
     def get_current(self):
-        if self.__iter is None: return None
-        elif self.is_empty(): return None
-        else: return self[self.__iter][0]
+        if self.__iter is None:
+            return None
+        elif self.is_empty():
+            return None
+        else:
+            return self[self.__iter][0]
 
     @property
     def get_current_path(self):
-        if self.__iter is None: return None
-        elif self.is_empty(): return None
-        else: return self[self.__iter].path
+        if self.__iter is None:
+            return None
+        elif self.is_empty():
+            return None
+        else:
+            return self[self.__iter].path
 
     @property
     def get_current_iter(self):
-        if self.__iter is None: return None
-        elif self.is_empty(): return None
-        else: return self.__iter
+        if self.__iter is None:
+            return None
+        elif self.is_empty():
+            return None
+        else:
+            return self.__iter
 
     def is_empty(self):
         return not len(self)
@@ -256,11 +274,12 @@ class DuplicatesTreeModel(gtk.TreeStore):
             object, str, str, str, str, str, str, str)
 
 
-class DuplicateDialog(gtk.Window):
+class DuplicateDialog(Gtk.Window):
     """Main dialog for browsing duplicate results"""
 
     def __quit(self, widget=None, response=None):
-        if response == gtk.RESPONSE_OK or response == gtk.RESPONSE_CLOSE:
+        if response == Gtk.ResponseType.OK or \
+                response == Gtk.ResponseType.CLOSE:
             print_d("Exiting plugin on user request...", self)
         self.finished = True
         self.destroy()
@@ -268,9 +287,9 @@ class DuplicateDialog(gtk.Window):
 
     def __songs_popup_menu(self, songlist):
         path, col = songlist.get_cursor()
-        menu = songlist.Menu(library)
+        menu = songlist.Menu(app.library)
         if menu is not None:
-            return songlist.popup_menu(menu, 0, gtk.get_current_event_time())
+            return songlist.popup_menu(menu, 0, Gtk.get_current_event_time())
 
     def __init__(self, model):
         songs_text = ngettext("%d duplicate group", "%d duplicate groups",
@@ -282,52 +301,75 @@ class DuplicateDialog(gtk.Window):
         self.finished = False
         self.set_default_size(960, 480)
         self.set_border_width(6)
-        swin = gtk.ScrolledWindow()
-        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        swin.set_shadow_type(gtk.SHADOW_IN)
+        swin = Gtk.ScrolledWindow()
+        swin.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        swin.set_shadow_type(Gtk.ShadowType.IN)
         # Set up the browser view
         view = DuplicateSongsView(model)
+
+        def cell_text(column, cell, model, iter_, index):
+            text = model[iter_][index]
+            cell.markup = text
+            cell.set_property("markup", text)
+
         # Set up the columns
         for i, (tag, f) in enumerate(DuplicatesTreeModel.TAG_MAP):
-            e = (pango.ELLIPSIZE_START if tag == '~filename'
-                else pango.ELLIPSIZE_END)
-            col = gtk.TreeViewColumn(util.tag(tag),
-                gobject.new(gtk.CellRendererText, ellipsize=e),
-                markup=i + 1)
+            e = (Pango.EllipsizeMode.START if tag == '~filename'
+                else Pango.EllipsizeMode.END)
+            render = Gtk.CellRendererText()
+            render.set_property("ellipsize", e)
+            col = Gtk.TreeViewColumn(util.tag(tag), render)
             # Numeric columns are better smaller here.
             if tag.startswith("~#"):
                 col.set_fixed_width(80)
-                col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+                col.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
             else:
                 col.set_expand(True)
-                col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+                col.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
             col.set_resizable(True)
+            col.set_cell_data_func(render, cell_text, i + 1)
             view.append_column(col)
 
         view.connect('popup-menu', self.__songs_popup_menu)
         swin.add(view)
         # A basic information area
-        hbox = gtk.HBox(spacing=6)
-        label = gtk.Label(_("Duplicate key expression is '%s'") %
-                Duplicates.get_key_expression())
-        hbox.pack_start(label)
-        close = gtk.Button(stock=gtk.STOCK_CLOSE)
-        close.connect('clicked', self.__quit)
-        hbox.pack_start(close, expand=False)
+        hbox = Gtk.HBox(spacing=6)
 
-        vbox = gtk.VBox(spacing=6)
-        vbox.pack_start(swin)
-        vbox.pack_start(hbox, expand=False)
+        def expand_all(*args):
+            model = view.get_model()
+            for row in model:
+                if view.row_expanded(row.path):
+                    for row in model:
+                        view.collapse_row(row.path)
+                    break
+            else:
+                for row in model:
+                    view.expand_row(row.path, False)
+
+        expand = Gtk.Button(_("Collapse / Expand all"))
+        expand.connect_object("clicked", expand_all, view)
+        hbox.pack_start(expand, False, True, 0)
+
+        label = Gtk.Label(label=_("Duplicate key expression is '%s'") %
+                Duplicates.get_key_expression())
+        hbox.pack_start(label, True, True, 0)
+        close = Gtk.Button(stock=Gtk.STOCK_CLOSE)
+        close.connect('clicked', self.__quit)
+        hbox.pack_start(close, False, True, 0)
+
+        vbox = Gtk.VBox(spacing=6)
+        vbox.pack_start(swin, True, True, 0)
+        vbox.pack_start(hbox, False, True, 0)
         self.add(vbox)
         self.show_all()
 
 
-class Duplicates(SongsMenuPlugin):
+class Duplicates(SongsMenuPlugin, PluginConfigMixin):
     PLUGIN_ID = 'Duplicates'
     PLUGIN_NAME = _('Duplicates Browser')
     PLUGIN_DESC = _('Find and browse similarly tagged versions of songs.')
-    PLUGIN_ICON = gtk.STOCK_MEDIA_PLAY
-    PLUGIN_VERSION = "0.6"
+    PLUGIN_ICON = Gtk.STOCK_MEDIA_PLAY
+    PLUGIN_VERSION = "0.7"
 
     MIN_GROUP_SIZE = 2
     _CFG_KEY_KEY = "key_expression"
@@ -345,69 +387,35 @@ class Duplicates(SongsMenuPlugin):
     # Faster than a speeding bullet
     __trans = string.maketrans("", "")
 
-    @classmethod
-    def _cfg_key(cls, key):
-        return "%s_%s" % (__name__, key)
-
-    @classmethod
-    def cfg_get(cls, name, default=None):
-        try:
-            key = cls._cfg_key(name)
-            return config.get("plugins", key)
-        except (ValueError, ConfigParser.Error):
-            print_w("Duplicates config entry for '%s' not found."
-                    "Defaulting to  '%s'" % (key, default))
-            return default
-
-    @classmethod
-    def cfg_get_bool(cls, key, default=True):
-        if key in cls.__cfg_cache: return bool(cls.__cfg_cache[key])
-        try:
-            key = cls._cfg_key(key)
-            cls.__cfg_cache[key] = bool
-            return bool(config.getboolean("plugins", key))
-        except (ValueError, ConfigParser.Error, TypeError):
-            print_w("Duplicates config entry for '%s' not found."
-                    "Defaulting to  '%s'" % (key, default))
-            return default
-
-
-    @classmethod
-    def cfg_set(cls, name, value):
-        cls.key_expression = value
-        key = cls._cfg_key(name)
-        print_d("Setting %s to '%s'" % (key,value))
-        config.set("plugins", key, value)
-
     @classmethod
     def get_key_expression(cls):
         if not cls.key_expression:
             cls.key_expression = (
-                    cls.cfg_get(cls._CFG_KEY_KEY, cls.__DEFAULT_KEY_VALUE))
+                    cls.config_get(cls._CFG_KEY_KEY, cls.__DEFAULT_KEY_VALUE))
         return cls.key_expression
 
     @classmethod
     def PluginPreferences(cls, window):
         def key_changed(entry):
-            #print_d("setting to %s" % entry.get_text().strip())
-            cls.cfg_set(cls._CFG_KEY_KEY, entry.get_text().strip())
+            cls.key_expression = None
+            cls.config_set(cls._CFG_KEY_KEY, entry.get_text().strip())
 
-        vb = gtk.VBox(spacing=10)
+        vb = Gtk.VBox(spacing=10)
         vb.set_border_width(0)
-        hbox = gtk.HBox(spacing=6)
+        hbox = Gtk.HBox(spacing=6)
         # TODO: construct a decent validator and use ValidatingEntry
         e = UndoEntry()
         e.set_text(cls.get_key_expression())
         e.connect("changed", key_changed)
         e.set_tooltip_markup("Accepts QL tag expressions like "
                 "<tt>~artist~title</tt> or <tt>musicbrainz_track_id</tt>")
-        lbl = gtk.Label(_("_Group duplicates by:"))
+        lbl = Gtk.Label(label=_("_Group duplicates by:"))
         lbl.set_mnemonic_widget(e)
         lbl.set_use_underline(True)
-        hbox.pack_start(lbl, expand=False)
-        hbox.pack_start(e, expand=False)
+        hbox.pack_start(lbl, False, True, 0)
+        hbox.pack_start(e, False, True, 0)
         frame = qltk.Frame(label=_("Duplicate Key"), child=hbox)
-        vb.pack_start(frame, expand=True)
+        vb.pack_start(frame, True, True, 0)
 
         # Matching Option
         toggles = [
@@ -416,14 +424,14 @@ class Duplicates(SongsMenuPlugin):
             (cls._CFG_REMOVE_PUNCTUATION, _("Remove _Punctuation")),
             (cls._CFG_CASE_INSENSITIVE, _("Case _Insensitive")),
         ]
-        vb2=gtk.VBox(spacing=6)
+        vb2 = Gtk.VBox(spacing=6)
         for key, label in toggles:
-            ccb = ConfigCheckButton(label, 'plugins', cls._cfg_key(key))
-            ccb.set_active(cls.cfg_get_bool(key))
-            vb2.pack_start(ccb)
+            ccb = ConfigCheckButton(label, 'plugins', cls._config_key(key))
+            ccb.set_active(cls.config_get_bool(key))
+            vb2.pack_start(ccb, True, True, 0)
 
         frame = qltk.Frame(label=_("Matching options"), child=vb2)
-        vb.pack_start(frame, expand=False)
+        vb.pack_start(frame, False, True, 0)
 
         vb.show_all()
         return vb
@@ -436,13 +444,13 @@ class Duplicates(SongsMenuPlugin):
     @classmethod
     def get_key(cls, song):
         key = song(cls.get_key_expression())
-        if cls.cfg_get_bool(cls._CFG_REMOVE_DIACRITICS):
+        if cls.config_get_bool(cls._CFG_REMOVE_DIACRITICS):
             key = cls.remove_accents(key)
-        if cls.cfg_get_bool(cls._CFG_CASE_INSENSITIVE):
+        if cls.config_get_bool(cls._CFG_CASE_INSENSITIVE):
             key = key.lower()
-        if cls.cfg_get_bool(cls._CFG_REMOVE_PUNCTUATION):
+        if cls.config_get_bool(cls._CFG_REMOVE_PUNCTUATION):
             key = str(key).translate(cls.__trans, string.punctuation)
-        if cls.cfg_get_bool(cls._CFG_REMOVE_WHITESPACE):
+        if cls.config_get_bool(cls._CFG_REMOVE_WHITESPACE):
             key = "_".join(key.split())
         return key
 
@@ -461,18 +469,17 @@ class Duplicates(SongsMenuPlugin):
             elif key:
                 groups[key] = set([song._song])
 
-        for song in library:
+        for song in app.library:
             key = self.get_key(song)
             if key in groups:
                 groups[key].add(song)
 
         # Now display the grouped duplicates
         for (key, children) in groups.items():
-            if len(children) < self.MIN_GROUP_SIZE: continue
-
+            if len(children) < self.MIN_GROUP_SIZE:
+                continue
             # The parent (group) label
             model.add_group(key, children)
 
         dialog = DuplicateDialog(model)
         dialog.show()
-
