@@ -1,13 +1,10 @@
-# Copyright 2011 Christoph Reiter
+# Copyright 2011,2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import gst
-import gobject
-import gtk
-import pango
+from gi.repository import Gtk, GObject, Gst, Pango, GLib
 
 import threading
 import urllib
@@ -23,12 +20,11 @@ from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.msg import ErrorMessage
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 
-if not gst.element_factory_find("chromaprint"):
+if not Gst.ElementFactory.find("chromaprint"):
     from quodlibet import plugins
-    if not hasattr(plugins, "PluginImportException"):
-        raise gst.PluginNotFoundError("chromaprint")
     raise plugins.PluginImportException("Couldn't find gst-chromaprint.")
 
+
 def get_num_threads():
     # multiprocessing is >= 2.6.
     # Default to 2 threads if cpu_count isn't implemented for the current arch
@@ -36,9 +32,11 @@ def get_num_threads():
     try:
         import multiprocessing
         threads = multiprocessing.cpu_count()
-    except (ImportError, NotImplementedError): threads = 2
+    except (ImportError, NotImplementedError):
+        threads = 2
     return threads
 
+
 class FingerPrintPipeline(threading.Thread):
     def __init__(self, pool, song, ofa):
         super(FingerPrintPipeline, self).__init__()
@@ -56,69 +54,69 @@ class FingerPrintPipeline(threading.Thread):
 
     def run(self):
         # pipeline
-        pipe = gst.Pipeline("pipe")
-
-        # decodebin2 got declared stable with 0.10.31
-        # https://bugzilla.gnome.org/show_bug.cgi?id=624949
-        use_decodebin2 = gst.version() >= (0, 10, 31)
+        pipe = Gst.Pipeline()
 
         # decode part
-        filesrc = gst.element_factory_make("filesrc")
+        filesrc = Gst.ElementFactory.make("filesrc", None)
         pipe.add(filesrc)
-        if use_decodebin2:
-            decode = gst.element_factory_make("decodebin2")
-        else:
-            decode = gst.element_factory_make("decodebin")
+
+        decode = Gst.ElementFactory.make("decodebin", None)
         pipe.add(decode)
-        gst.element_link_many(filesrc, decode)
+        Gst.Element.link(filesrc, decode)
 
         # convert to right format
-        elements = map(gst.element_factory_make,
-                       ["audioconvert", "audioresample"])
-        map(pipe.add, elements)
-        gst.element_link_many(*elements)
-        convert, resample = elements
+        convert = Gst.ElementFactory.make("audioconvert", None)
+        resample = Gst.ElementFactory.make("audioresample", None)
+        pipe.add(convert)
+        pipe.add(resample)
+        Gst.Element.link(convert, resample)
 
         # ffdec_mp3 got disabled in gstreamer
         # (for a reason they don't remember), reenable it..
         # http://cgit.freedesktop.org/gstreamer/gst-ffmpeg/commit/
         # ?id=2de5aaf22d6762450857d644e815d858bc0cce65
-        ffdec_mp3 = gst.element_factory_find("ffdec_mp3")
+        ffdec_mp3 = Gst.ElementFactory.find("ffdec_mp3")
         if ffdec_mp3:
-            ffdec_mp3.set_rank(gst.RANK_MARGINAL)
+            ffdec_mp3.set_rank(Gst.Rank.MARGINAL)
 
         # decodebin creates pad, we link it
-        if use_decodebin2:
-            decode.connect_object("pad-added", self.__new_decoded_pad, convert)
-            decode.connect("autoplug-sort", self.__sort_decoders)
-        else:
-            decode.connect_object(
-                "new-decoded-pad", self.__new_decoded_pad, convert)
+        decode.connect_object("pad-added", self.__new_decoded_pad, convert)
+        decode.connect("autoplug-sort", self.__sort_decoders)
 
         chroma_src = resample
 
-        use_ofa = self.__ofa and gst.element_factory_find("ofa")
+        use_ofa = self.__ofa and Gst.ElementFactory.find("ofa")
 
         if use_ofa:
             # create a tee and one queue for chroma
-            elements = map(gst.element_factory_make, ["tee", "queue"])
-            map(pipe.add, elements)
-            gst.element_link_many(resample, *elements)
-            tee, chroma_queue = elements
+            tee = Gst.ElementFactory.make("tee", None)
+            chroma_queue = Gst.ElementFactory.make("queue", None)
+            pipe.add(tee)
+            pipe.add(chroma_queue)
+            Gst.Element.link(resample, tee)
+            Gst.Element.link(tee, chroma_queue)
 
             chroma_src = chroma_queue
 
-            elements = map(gst.element_factory_make,
-                           ["queue", "ofa", "fakesink"])
-            map(pipe.add, elements)
-            gst.element_link_many(tee, *elements)
-            ofa = elements[1]
+            ofa_queue = Gst.ElementFactory.make("queue", None)
+            ofa = Gst.ElementFactory.make("ofa", None)
+            fake = Gst.ElementFactory.make("fakesink", None)
+            pipe.add(ofa_queue)
+            pipe.add(ofa)
+            pipe.add(fake)
+
+            Gst.Element.link(tee, ofa_queue)
+            Gst.Element.link(ofa_queue, ofa)
+            Gst.Element.link(ofa, fake)
             self.__todo.append(ofa)
 
-        elements = map(gst.element_factory_make, ["chromaprint", "fakesink"])
-        map(pipe.add, elements)
-        gst.element_link_many(chroma_src, *elements)
-        chroma = elements[0]
+        chroma = Gst.ElementFactory.make("chromaprint", None)
+        fake2 = Gst.ElementFactory.make("fakesink", None)
+        pipe.add(chroma)
+        pipe.add(fake2)
+
+        Gst.Element.link(chroma_src, chroma)
+        Gst.Element.link(chroma, fake2)
         self.__todo.append(chroma)
 
         filesrc.set_property("location", self.__song["~filename"])
@@ -132,35 +130,35 @@ class FingerPrintPipeline(threading.Thread):
 
         # get it started
         self.__cv.acquire()
-        pipe.set_state(gst.STATE_PLAYING)
+        pipe.set_state(Gst.State.PLAYING)
 
-        result = pipe.get_state()[0]
-        if result == gst.STATE_CHANGE_FAILURE:
+        result = pipe.get_state(timeout=Gst.SECOND / 2)[0]
+        if result == Gst.StateChangeReturn.FAILURE:
             # something failed, error message kicks in before, so check
             # for shutdown
             if not self.__shutdown:
                 self.__shutdown = True
-                gobject.idle_add(self.__pool._callback, self.__song,
+                GLib.idle_add(self.__pool._callback, self.__song,
                     None, "Error", self)
         elif not self.__shutdown:
             # GStreamer probably knows song durations better than we do.
             # (and it's more precise for PUID lookup)
             # In case this fails, we insert the mutagen value later
             # (this only works in active playing state)
-            try: d = pipe.query_duration(gst.FORMAT_TIME)[0]
-            except gst.QueryError: pass
-            else: self.__fingerprints["length"] = d / gst.MSECOND
+            ok, d = pipe.query_duration(Gst.Format.TIME)
+            if ok:
+                self.__fingerprints["length"] = d / Gst.MSECOND
 
             self.__cv.wait()
         self.__cv.release()
 
         # clean up
         bus.remove_signal_watch()
-        pipe.set_state(gst.STATE_NULL)
+        pipe.set_state(Gst.State.NULL)
 
         # we need to make sure the state change has finished, before
         # we can return and hand it over to the python GC
-        pipe.get_state()
+        pipe.get_state(timeout=Gst.SECOND / 2)
 
     def stop(self):
         self.__shutdown = True
@@ -182,64 +180,56 @@ class FingerPrintPipeline(threading.Thread):
 
         def set_prio(x):
             i, f = x
-            i = {"mad": -1, "ffdec_mp3": -2, "mpg123": -3}.get(f.get_name(), i)
+            i = {
+                "mad": -1,
+                "ffdec_mp3": -2,
+                "mpg123audiodec": -3
+            }.get(f.get_name(), i)
             return (i, f)
 
         return zip(*sorted(map(set_prio, enumerate(factories))))[1]
 
     def __new_decoded_pad(self, convert, pad, *args):
-        pad.link(convert.get_pad("sink"))
+        pad.link(convert.get_static_pad("sink"))
 
     def __bus_message(self, bus, message, chroma, ofa):
         error = None
-        if message.type == gst.MESSAGE_TAG:
-            if message.src == chroma:
-                tags = message.parse_tag()
-                key = "chromaprint-fingerprint"
-                if key in tags.keys():
-                    if chroma in self.__todo:
-                        self.__todo.remove(chroma)
-                    self.__fingerprints["chromaprint"] = tags[key]
-            elif message.src == ofa:
-                tags = message.parse_tag()
-                # https://bugzilla.gnome.org/show_bug.cgi?id=656641
-                # (which promptly got fixed)
-                # Because libofa fails if the first 135 seconds are silent
-                # gst-ofa will emit an empty tag list and then segfault
-                # on EOS. It will also segfault if there is no data flow at
-                # all before EOS, but there is nothing we can do about it
-                # and that shouldn't happen very often.
-                #
-                # As a workaround we mark it done whenever it emits something
-                # and since it should come after chromaprint (chromaprint
-                # defaults to 120, while ofa to 135 seconds) the pipeline
-                # should stop after this.
+        if message.type == Gst.MessageType.TAG:
+            tags = message.parse_tag()
+
+            ok, value = tags.get_string("chromaprint-fingerprint")
+            if ok:
+                if chroma in self.__todo:
+                    self.__todo.remove(chroma)
+                self.__fingerprints["chromaprint"] = value
+
+            ok, value = tags.get_string("ofa-fingerprint")
+            if ok:
                 if ofa in self.__todo:
                     self.__todo.remove(ofa)
-
-                key = "ofa-fingerprint"
-                if key in tags.keys():
-                    self.__fingerprints["ofa"] = tags[key]
-        elif message.type == gst.MESSAGE_EOS:
+                self.__fingerprints["ofa"] = value
+        elif message.type == Gst.MessageType.EOS:
             error = "EOS"
-        elif message.type == gst.MESSAGE_ERROR:
+        elif message.type == Gst.MessageType.ERROR:
             error = str(message.parse_error()[0])
+
         if not self.__shutdown and (not self.__todo or error):
-            gobject.idle_add(self.__pool._callback, self.__song,
+            GLib.idle_add(self.__pool._callback, self.__song,
                 self.__fingerprints, error, self)
             self.__shutdown = True
             self.__cv.acquire()
             self.__cv.notify()
             self.__cv.release()
 
-class FingerPrintThreadPool(gobject.GObject):
+
+class FingerPrintThreadPool(GObject.GObject):
     __gsignals__ = {
         "fingerprint-done": (
-            gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (object, object)),
+            GObject.SignalFlags.RUN_LAST, None, (object, object)),
         "fingerprint-started": (
-            gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (object,)),
+            GObject.SignalFlags.RUN_LAST, None, (object,)),
         "fingerprint-error": (
-            gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (object, object)),
+            GObject.SignalFlags.RUN_LAST, None, (object, object)),
         }
 
     def __init__(self, max_workers):
@@ -268,7 +258,8 @@ class FingerPrintThreadPool(gobject.GObject):
         # make sure everythin is gone before starting new ones.
         thread.join()
         self.__threads.remove(thread)
-        if self.__stopped: return
+        if self.__stopped:
+            return
         if not error:
             self.emit("fingerprint-done", song, result)
         else:
@@ -278,6 +269,7 @@ class FingerPrintThreadPool(gobject.GObject):
             self.__threads.append(FingerPrintPipeline(self, song, ofa))
             self.emit("fingerprint-started", song)
 
+
 class MusicDNSThread(threading.Thread):
     INTERVAL = 1500
     URL = "http://ofa.musicdns.org/ofa/1/track"
@@ -328,8 +320,10 @@ class MusicDNSThread(threading.Thread):
             error = "urllib error"
         else:
             xml = response.read()
-            try: dom = parseString(xml)
-            except: error = "xml error"
+            try:
+                dom = parseString(xml)
+            except:
+                error = "xml error"
             else:
                 puids = dom.getElementsByTagName("puid")
                 if puids and puids[0].hasAttribute("id"):
@@ -340,20 +334,23 @@ class MusicDNSThread(threading.Thread):
 
     def run(self):
         self.__sem.release()
-        gobject.timeout_add(self.INTERVAL, self.__inc_sem)
+        GLib.timeout_add(self.INTERVAL, self.__inc_sem)
 
-        items = [(s,d) for s,d in self.__fingerprints.iteritems() if "ofa" in d]
+        items = [(s, d) for s, d in self.__fingerprints.iteritems()
+                 if "ofa" in d]
         for i, (song, data) in enumerate(items):
             self.__sem.acquire()
-            if self.__stopped: return
+            if self.__stopped:
+                return
 
             puid = self.__get_puid(data["ofa"], data["length"])
-            if puid: data["puid"] = puid
+            if puid:
+                data["puid"] = puid
 
-            gobject.idle_add(self.__progress_cb, song,
+            GLib.idle_add(self.__progress_cb, song,
                 float(i + 1) / len(items))
 
-        gobject.idle_add(self.__callback, self)
+        GLib.idle_add(self.__callback, self)
 
         # stop sem increment
         self.__stopped = True
@@ -367,6 +364,7 @@ class MusicDNSThread(threading.Thread):
         self.__stopped = True
         self.__sem.release()
 
+
 class AcoustidSubmissionThread(threading.Thread):
     INTERVAL = 1500
     URL = "http://api.acoustid.org/v2/submit"
@@ -386,7 +384,8 @@ class AcoustidSubmissionThread(threading.Thread):
 
     def __send(self, urldata):
         self.__sem.acquire()
-        if self.__stopped: return
+        if self.__stopped:
+            return
 
         self.__done += len(urldata)
 
@@ -416,8 +415,10 @@ class AcoustidSubmissionThread(threading.Thread):
             error = "urllib error"
         else:
             xml = response.read()
-            try: dom = parseString(xml)
-            except: error = "xml error"
+            try:
+                dom = parseString(xml)
+            except:
+                error = "xml error"
             else:
                 status = dom.getElementsByTagName("status")
                 if not status or not status[0].childNodes or not \
@@ -428,16 +429,17 @@ class AcoustidSubmissionThread(threading.Thread):
             print_w("[fingerprint] " + _("Submission failed: ") + error)
 
         # emit progress
-        gobject.idle_add(self.__progress_cb,
-                float(self.__done)/len(self.__fingerprints))
+        GLib.idle_add(self.__progress_cb,
+                float(self.__done) / len(self.__fingerprints))
 
     def run(self):
         self.__sem.release()
-        gobject.timeout_add(self.INTERVAL, self.__inc_sem)
+        GLib.timeout_add(self.INTERVAL, self.__inc_sem)
 
         urldata = []
         for i, (song, data) in enumerate(self.__fingerprints.iteritems()):
-            if song in self.__invalid: continue
+            if song in self.__invalid:
+                continue
 
             track = {
                 "duration": int(round(data["length"] / 1000)),
@@ -457,7 +459,8 @@ class AcoustidSubmissionThread(threading.Thread):
             tuples = []
             for key, value in track.iteritems():
                 # this also dismisses 0.. which should be ok here.
-                if not value: continue
+                if not value:
+                    continue
                 # the postfixes don't have to start at a specific point,
                 # they just need to be different and numbers
                 key += ".%d" % i
@@ -473,12 +476,13 @@ class AcoustidSubmissionThread(threading.Thread):
                 self.__send(urldata)
                 urldata = []
 
-            if self.__stopped: return
+            if self.__stopped:
+                return
 
         if urldata:
             self.__send(urldata)
 
-        gobject.idle_add(self.__callback, self)
+        GLib.idle_add(self.__callback, self)
 
         # stop sem increment
         self.__stopped = True
@@ -492,6 +496,7 @@ class AcoustidSubmissionThread(threading.Thread):
         self.__stopped = True
         self.__sem.release()
 
+
 class FingerprintDialog(Window):
     def __init__(self, songs):
         super(FingerprintDialog, self).__init__()
@@ -499,35 +504,36 @@ class FingerprintDialog(Window):
         self.set_title(_("Submit Acoustic Fingerprints"))
         self.set_default_size(300, 0)
 
-        outer_box = gtk.VBox(spacing=12)
+        outer_box = Gtk.VBox(spacing=12)
 
-        box = gtk.VBox(spacing=6)
+        box = Gtk.VBox(spacing=6)
 
-        self.__label = label = gtk.Label()
+        self.__label = label = Gtk.Label()
         label.set_markup("<b>%s</b>" % _("Generating fingerprints:"))
         label.set_alignment(0, 0.5)
-        box.pack_start(label, expand=False)
+        box.pack_start(label, False, True, 0)
 
-        self.__bar = bar = gtk.ProgressBar()
+        self.__bar = bar = Gtk.ProgressBar()
         self.__set_fraction(0)
-        box.pack_start(bar, expand=False)
-        self.__label_song = label_song = gtk.Label()
+        box.pack_start(bar, False, True, 0)
+        self.__label_song = label_song = Gtk.Label()
         label_song.set_alignment(0, 0.5)
-        label_song.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
-        box.pack_start(label_song, expand=False)
+        label_song.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
+        box.pack_start(label_song, False, True, 0)
 
-        self.__stats = stats = gtk.Label()
+        self.__stats = stats = Gtk.Label()
         stats.set_alignment(0, 0.5)
-        expand = gtk.expander_new_with_mnemonic(_("_Details"))
-        align = gtk.Alignment(xalign=0.0, yalign=0.0, xscale=1.0, yscale=1.0)
+        expand = Gtk.Expander.new_with_mnemonic(_("_Details"))
+        align = Gtk.Alignment.new(0.0, 0.0, 1.0, 1.0)
         align.set_padding(6, 0, 6, 0)
         expand.add(align)
         align.add(stats)
+
         def expand_cb(expand, *args):
             self.resize(self.get_size()[0], 1)
         stats.connect("unmap", expand_cb)
 
-        box.pack_start(expand, expand=False, fill=False)
+        box.pack_start(expand, False, False, 0)
 
         self.__fp_results = {}
         self.__fp_done = 0
@@ -554,19 +560,19 @@ class FingerprintDialog(Window):
 
         pool = FingerPrintThreadPool(get_num_threads())
 
-        bbox = gtk.HButtonBox()
-        bbox.set_layout(gtk.BUTTONBOX_END)
+        bbox = Gtk.HButtonBox()
+        bbox.set_layout(Gtk.ButtonBoxStyle.END)
         bbox.set_spacing(6)
-        self.__submit = submit = Button(_("_Submit"), gtk.STOCK_APPLY)
+        self.__submit = submit = Button(_("_Submit"), Gtk.STOCK_APPLY)
         submit.set_sensitive(False)
         submit.connect('clicked', self.__submit_cb)
-        cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
+        cancel = Gtk.Button(stock=Gtk.STOCK_CANCEL)
         cancel.connect_object('clicked', self.__cancel_cb, pool)
-        bbox.pack_start(submit)
-        bbox.pack_start(cancel)
+        bbox.pack_start(submit, True, True, 0)
+        bbox.pack_start(cancel, True, True, 0)
 
-        outer_box.pack_start(box, expand=False)
-        outer_box.pack_start(bbox, expand=False)
+        outer_box.pack_start(box, False, True, 0)
+        outer_box.pack_start(bbox, False, True, 0)
 
         pool.connect('fingerprint-done', self.__fp_done_cb)
         pool.connect('fingerprint-error', self.__fp_error_cb)
@@ -592,8 +598,8 @@ class FingerprintDialog(Window):
         to_send = all - len(self.__invalid_songs)
         valid_fp = len(self.__fp_results)
 
-        text = _("Songs either need a <i><b>musicbrainz_trackid</b></i>, " \
-            "a <i><b>puid</b></i>\nor <i><b>artist</b></i> / " \
+        text = _("Songs either need a <i><b>musicbrainz_trackid</b></i>, "
+            "a <i><b>puid</b></i>\nor <i><b>artist</b></i> / "
             "<i><b>title</b></i> / <i><b>album</b></i> tags to get submitted.")
         text += _("\n\n<i>Fingerprints:</i> %d/%d") % (valid_fp, all)
         text += _("\n<i>Songs with MBIDs:</i> %d/%d") % (self.__mbids, all)
@@ -609,7 +615,7 @@ class FingerprintDialog(Window):
         for song, data in self.__fp_results.iteritems():
             artist = song("artist")
             title = song.get("title", "") # title falls back to filename
-            album  = song("album")
+            album = song("album")
             puid = song("puid") or data.get("puid", "")
             mbid = song("musicbrainz_trackid")
             if mbid or puid or (artist and title and album):
@@ -625,7 +631,7 @@ class FingerprintDialog(Window):
         frac = self.__fp_done / float(len(self.__songs))
         self.__set_fraction(frac)
         if self.__fp_done == len(self.__songs):
-            gobject.timeout_add(500, self.__start_puid)
+            GLib.timeout_add(500, self.__start_puid)
 
     def __fp_started_cb(self, pool, song):
         # increase by an amount smaller than one song, so that the user can
@@ -682,6 +688,7 @@ class FingerprintDialog(Window):
 
     def __cancel_cb(self, pool, *args):
         self.destroy()
+
         def idle_cancel():
             pool.stop()
             if self.__musicdns_thread:
@@ -690,7 +697,7 @@ class FingerprintDialog(Window):
                 self.__acoustid_thread.stop()
         # pool.stop can block a short time because the CV might be locked
         # during starting the pipeline -> idle_add -> no GUI blocking
-        gobject.idle_add(idle_cancel)
+        GLib.idle_add(idle_cancel)
 
     def __submit_cb(self, *args):
         self.__submit.set_sensitive(False)
@@ -708,26 +715,23 @@ class FingerprintDialog(Window):
         thread.join()
         self.__set_fraction(1.0)
         self.__show_final_stats()
-        gobject.timeout_add(500, self.destroy)
+        GLib.timeout_add(500, self.destroy)
+
 
 def get_api_key():
-    try:
-        return config.get("plugins", "fingerprint_acoustid_api_key")
-    except config.error:
-        return ""
+    return config.get("plugins", "fingerprint_acoustid_api_key", "")
+
 
 def get_puid_lookup():
-    try:
-        return config.get("plugins", "fingerprint_puid_lookup")
-    except config.error:
-        return "no_mbid"
+    return config.get("plugins", "fingerprint_puid_lookup", "no_mbid")
+
 
 class AcoustidSubmit(SongsMenuPlugin):
     PLUGIN_ID = "AcoustidSubmit"
     PLUGIN_NAME = _("Submit Acoustic Fingerprints")
     PLUGIN_DESC = _("Generates acoustic fingerprints using chromaprint and "
         "libofa and submits them to 'acoustid.org'")
-    PLUGIN_ICON = gtk.STOCK_CONNECT
+    PLUGIN_ICON = Gtk.STOCK_CONNECT
     PLUGIN_VERSION = "0.1"
 
     def plugin_songs(self, songs):
@@ -740,31 +744,32 @@ class AcoustidSubmit(SongsMenuPlugin):
 
     @classmethod
     def PluginPreferences(self, win):
-        box = gtk.VBox(spacing=12)
+        box = Gtk.VBox(spacing=12)
 
         # api key section
         def key_changed(entry, *args):
             config.set("plugins", "fingerprint_acoustid_api_key",
                 entry.get_text())
 
-        button = Button(_("Request API key"), gtk.STOCK_NETWORK)
+        button = Button(_("Request API key"), Gtk.STOCK_NETWORK)
         button.connect("clicked",
             lambda s: util.website("https://acoustid.org/api-key"))
-        key_box = gtk.HBox(spacing=6)
+        key_box = Gtk.HBox(spacing=6)
         entry = UndoEntry()
         entry.set_text(get_api_key())
         entry.connect("changed", key_changed)
-        label = gtk.Label(_("API _key:"))
+        label = Gtk.Label(label=_("API _key:"))
         label.set_use_underline(True)
         label.set_mnemonic_widget(entry)
-        key_box.pack_start(label, expand=False)
-        key_box.pack_start(entry)
-        key_box.pack_start(button, expand=False)
+        key_box.pack_start(label, False, True, 0)
+        key_box.pack_start(entry, True, True, 0)
+        key_box.pack_start(button, False, True, 0)
 
-        box.pack_start(Frame(_("Acoustid Web Service"), child=key_box))
+        box.pack_start(Frame(_("Acoustid Web Service"),
+                       child=key_box), True, True, 0)
 
         # puid lookup section
-        puid_box = gtk.VBox(spacing=6)
+        puid_box = Gtk.VBox(spacing=6)
         options = [
             ("no_mbid", _("If <i>_musicbrainz__trackid</i> is missing")),
             ("always", _("_Always")),
@@ -778,12 +783,13 @@ class AcoustidSubmit(SongsMenuPlugin):
         start_value = get_puid_lookup()
         radio = None
         for value, text in options:
-            radio = gtk.RadioButton(group=radio, label=text)
-            radio.child.set_use_markup(True)
+            radio = Gtk.RadioButton(group=radio, label=text,
+                                    use_underline=True)
+            radio.get_child().set_use_markup(True)
             radio.set_active(value == start_value)
             radio.connect("toggled", config_changed, value)
-            puid_box.pack_start(radio)
+            puid_box.pack_start(radio, True, True, 0)
 
-        box.pack_start(Frame(_("PUID Lookup"), child=puid_box))
+        box.pack_start(Frame(_("PUID Lookup"), child=puid_box), True, True, 0)
 
         return box
