@@ -4,10 +4,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import os
-import re
-
-import gtk
+from gi.repository import Gtk
 
 from quodlibet import config
 from quodlibet.util import tag, escape
@@ -38,47 +35,52 @@ HTML = '''<?xml version="1.0" encoding="UTF-8"?>
 </html>
 '''
 
+
+def to_html(songs):
+    cols = config.get_columns()
+
+    cols_s = ""
+    for col in cols:
+        cols_s += '<th>%s</th>' % tag(col)
+
+    songs_s = ""
+    for song in songs:
+        s = '<tr>'
+        for col in cols:
+            col = {"~#rating": "~rating", "~#length": "~length"}.get(
+                col, col)
+            s += '\n<td>%s</td>' % (
+                escape(unicode(song.comma(col))) or '&nbsp;')
+        s += '</tr>'
+        songs_s += s
+
+    return HTML % {'headers': cols_s, 'songs': songs_s}
+
+
 class ExportToHTML(SongsMenuPlugin):
     PLUGIN_ID = "Export to HTML"
     PLUGIN_NAME = _("Export to HTML")
     PLUGIN_DESC = _("Export the selected song list to HTML.")
-    PLUGIN_ICON = gtk.STOCK_CONVERT
+    PLUGIN_ICON = Gtk.STOCK_CONVERT
     PLUGIN_VERSION = "0.17"
 
     def plugin_songs(self, songs):
-        if not songs: return
+        if not songs:
+            return
 
-        chooser = gtk.FileChooserDialog(
+        chooser = Gtk.FileChooserDialog(
             title="Export to HTML",
-            action=gtk.FILE_CHOOSER_ACTION_SAVE,
-            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
-                     gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-        chooser.set_default_response(gtk.RESPONSE_ACCEPT)
+            action=Gtk.FileChooserAction.SAVE,
+            buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
+                     Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
+        chooser.set_default_response(Gtk.ResponseType.ACCEPT)
         resp = chooser.run()
-        if resp != gtk.RESPONSE_ACCEPT:
+        if resp != Gtk.ResponseType.ACCEPT:
             chooser.destroy()
             return
 
         fn = chooser.get_filename()
         chooser.destroy()
 
-        cols = config.get("settings", "headers").split()
-
-        cols_s = ""
-        for col in cols:
-            cols_s += '<th>%s</th>' % tag(col)
-
-        songs_s = ""
-        for song in songs:
-            s = '<tr>'
-            for col in cols:
-                col = {"~#rating":"~rating", "~#length":"~length"}.get(
-                    col, col)
-                s += '\n<td>%s</td>' % (
-                    escape(unicode(song.comma(col))) or '&nbsp;')
-            s += '</tr>'
-            songs_s += s
-
-        f = open(fn, 'w')
-        f.write((HTML % {'headers': cols_s, 'songs': songs_s}).encode('utf-8'))
-        f.close()
+        with open(fn, "wb") as f:
+            f.write(to_html(songs).encode("utf-8"))
