@@ -9,12 +9,13 @@
 
 import os
 
-import gtk
+from gi.repository import Gtk
 
 from quodlibet import util, qltk
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.const import HOME as lastfolder
 
+
 if hasattr(os.path, 'relpath'):
     relpath = os.path.relpath
 else:
@@ -31,11 +32,12 @@ else:
         # Work out how much of the filepath is shared by start and path.
         i = len(os.path.commonprefix([start_list, path_list]))
 
-        rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
+        rel_list = [os.path.pardir] * (len(start_list) - i) + path_list[i:]
         if not rel_list:
             return os.path.curdir
         return os.path.join(*rel_list)
 
+
 class PlaylistExport(SongsMenuPlugin):
     PLUGIN_ID = 'Playlist Export'
     PLUGIN_NAME = _('Playlist Export')
@@ -45,20 +47,20 @@ class PlaylistExport(SongsMenuPlugin):
 
     def plugin_songs(self, songs):
         global lastfolder
-        dialog = gtk.FileChooserDialog(self.PLUGIN_NAME,
+        dialog = Gtk.FileChooserDialog(self.PLUGIN_NAME,
             None,
-            gtk.FILE_CHOOSER_ACTION_SAVE,
-            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-            gtk.STOCK_SAVE, gtk.RESPONSE_OK))
-        dialog.set_default_response(gtk.RESPONSE_OK)
+            Gtk.FileChooserAction.SAVE,
+            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+            Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+        dialog.set_default_response(Gtk.ResponseType.OK)
 
-        ffilter = gtk.FileFilter()
+        ffilter = Gtk.FileFilter()
         ffilter.set_name("m3u")
         ffilter.add_mime_type("audio/x-mpegurl")
         ffilter.add_pattern("*.m3u")
         dialog.add_filter(ffilter)
 
-        ffilter = gtk.FileFilter()
+        ffilter = Gtk.FileFilter()
         ffilter.set_name("pls")
         ffilter.add_mime_type("audio/x-scpls")
         ffilter.add_pattern("*.pls")
@@ -66,11 +68,11 @@ class PlaylistExport(SongsMenuPlugin):
 
         dialog.set_current_folder(lastfolder)
 
-        diag_cont = dialog.child
-        hbox_path = gtk.HBox()
-        combo_path = gtk.combo_box_new_text()
-        hbox_path.pack_end(combo_path, False, False, padding=6)
-        diag_cont.pack_start(hbox_path, False, False)
+        diag_cont = dialog.get_child()
+        hbox_path = Gtk.HBox()
+        combo_path = Gtk.ComboBoxText()
+        hbox_path.pack_end(combo_path, False, False, 6)
+        diag_cont.pack_start(hbox_path, False, False, 0)
         diag_cont.show_all()
 
         map(combo_path.append_text, [_("Relative path"), _("Absolute path")])
@@ -78,7 +80,7 @@ class PlaylistExport(SongsMenuPlugin):
 
         response = dialog.run()
 
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             file_path = dialog.get_filename()
             dir_path = os.path.dirname(file_path)
 
@@ -111,7 +113,7 @@ class PlaylistExport(SongsMenuPlugin):
         files = []
         for song in songs:
             f = {}
-            if song.has_key("~uri"):
+            if "~uri" in song:
                 f['path'] = song('~filename')
                 f['title'] = song("title")
                 f['length'] = -1
@@ -121,7 +123,7 @@ class PlaylistExport(SongsMenuPlugin):
                     path = relpath(path, dir_path)
                 f['path'] = path
                 f['title'] = "%s - %s" % (
-                    song('~people').replace("\n",", "),
+                    song('~people').replace("\n", ", "),
                     song('~title~version'))
                 f['length'] = song('~#length')
             files.append(f)
