@@ -6,18 +6,21 @@
 
 from urllib import quote
 
-import gtk
+from gi.repository import Gtk
 
 from quodlibet import config
 from quodlibet.util import website
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 
 WIKI_URL = "http://%s.wikipedia.org/wiki/"
-try: config.get("plugins", __name__)
-except: config.set("plugins", __name__, "en")
+try:
+    config.get("plugins", __name__)
+except:
+    config.set("plugins", __name__, "en")
+
 
 class WikiSearch(object):
-    PLUGIN_ICON = gtk.STOCK_OPEN
+    PLUGIN_ICON = Gtk.STOCK_OPEN
     PLUGIN_VERSION = '0.14'
 
     def changed(self, e):
@@ -25,15 +28,15 @@ class WikiSearch(object):
     changed = classmethod(changed)
 
     def PluginPreferences(self, parent):
-        hb = gtk.HBox(spacing=3)
+        hb = Gtk.HBox(spacing=3)
         hb.set_border_width(6)
-        e = gtk.Entry(2)
+        e = Gtk.Entry(max_length=2)
         e.set_width_chars(3)
         e.set_text(config.get('plugins', __name__))
         e.connect('changed', self.changed)
-        hb.pack_start(gtk.Label("Search at http://"), expand=False)
-        hb.pack_start(e, expand=False)
-        hb.pack_start(gtk.Label(".wikipedia.org"), expand=False)
+        hb.pack_start(Gtk.Label("Search at http://"), False, True, 0)
+        hb.pack_start(e, False, True, 0)
+        hb.pack_start(Gtk.Label(".wikipedia.org"), False, True, 0)
         hb.show_all()
         return hb
     PluginPreferences = classmethod(PluginPreferences)
@@ -44,11 +47,13 @@ class WikiSearch(object):
             a = quote(str(a).title().replace(' ', '_'))
             website(WIKI_URL % config.get('plugins', __name__) + a)
 
+
 class WikiArtist(WikiSearch, SongsMenuPlugin):
     PLUGIN_ID = 'Search artist in Wikipedia'
     PLUGIN_NAME = _('Search artist in Wikipedia')
     k = 'artist'
 
+
 class WikiAlbum(WikiSearch, SongsMenuPlugin):
     PLUGIN_ID = 'Search album in Wikipedia'
     PLUGIN_NAME = _('Search album in Wikipedia')
